"""Internal utilities."""
import random
import string
from typing import Any

from google.protobuf.message import Message
from google.protobuf.text_format import MessageToString

MESSAGE_MAX_LENGTH = 256


def get_call_id() -> str:
    """Return random call ID for logging."""
    return ''.join(random.choice(string.ascii_letters + string.digits) for _ in range(4))  # nosec


def format_message(message: Any) -> str:
    """Format a message to string.

    If argument is a gRPC Message, format it nicely.
    If it's another object, use simple str conversion.
    In any case the output has limited length.
    """
    if isinstance(message, Message):
        out = MessageToString(message, as_one_line=True)
    else:
        out = str(message)
    if len(out) > MESSAGE_MAX_LENGTH:
        return out[:MESSAGE_MAX_LENGTH - 3] + '...'
    else:
        return out


class lazy_format_message:
    """Lazy format a message to string.

    If argument is a gRPC Message, format it nicely.
    If it's another object, use simple str conversion.
    In any case the output has limited length.
    """

    def __init__(self, message: Any):
        self._message = message

    def __repr__(self) -> str:
        return format_message(self._message)
